library airtable;

import 'package:http/http.dart' as http;
import 'dart:convert';

///Simple Airtable API for quering and filtering in Dart
class Airtable {
  static final Airtable _airtableSingleton = new Airtable._internal();
  Airtable._internal();
  factory Airtable() => _airtableSingleton;

  String _apiKey, _projectBase, _baseURL = "https://api.airtable.com/v0/";

  ///Initialize the Airtable Library
  ///apiKey: The apiKey of the airtable
  ///projectBase: the base ID of the project
  static initilize(String apiKey, String projectBase) {
    _airtableSingleton._apiKey = apiKey;
    _airtableSingleton._projectBase = projectBase;
    _airtableSingleton._baseURL += projectBase;
  }

  ///Query elements of a specific table
  ///
  ///name: the name of the table
  ///maxRecords(Optional): the amout of items that are return from Airtable
  queryByTable(String name, {maxRecords: 100}) async => _query(_airtableSingleton
          ._baseURL +
      "/$name?api_key=${_airtableSingleton._apiKey}&maxRecords=$maxRecords");

  ///Querying an item from a specific table from the value of a field.
  ///
  ///tableName: The table were the item is located
  ///fieldName: The value of the field
  ///value: The value of the field
  queryByName(String tableName, String fieldName, String value,
          {maxRecords: 100}) async =>
      _query(_airtableSingleton._baseURL +
          "/$tableName?api_key=${_airtableSingleton._apiKey}&filterByFormula={$fieldName}=\"$value\"&maxRecords=$maxRecords");

  ///Querying an item by its id.
  ///
  ///tableName: the name of the table were the item is located
  ///id: id of the item
  queryByID(String tableName, String id) async =>
      _query(_airtableSingleton._baseURL +
          "/$tableName/$id?api_key=${_airtableSingleton._apiKey}");

  ///Creates an Item in a specified table
  ///
  ///tableName: The table were the new item is going to be created
  ///content: The description of the item.
  createItem(String tableName, Map<dynamic, dynamic> content) async {
    final response = await http.post(
        _airtableSingleton._baseURL +
            "/$tableName?api_key=${_airtableSingleton._apiKey}",
        body: json.encode(content),
        headers: {"Content-Type": "application/json"});
    final responseJson = jsonDecode(response.body);
    return responseJson;
  }

  ///id: the id of the item that is going to be updated.
  ///tableName: Where the the item is located
  ///updateItem:
  ///-key: the name of the field
  ///-value: the new value we are going to add or update.
  ///
  ///If the value in the field that is going to be updated is of type List then we are going to add the
  ///item to the list. If the value is a String we are going to update the value. Otherwise its going to
  ///return null
  updateItem(String id, String tableName, MapEntry updateItem) async {
    Map fields = (await queryByID(tableName, id))["fields"] ?? {};
    if (fields.containsKey(updateItem.key)) {
      MapEntry content;
      var field = (fields[updateItem.key]);
      if (field is List) {
        List listOfItems = field ?? [];
        listOfItems.add(updateItem.value);
        content = MapEntry(updateItem.key, listOfItems);
      } else if (field is String) {
        content = MapEntry(updateItem.key, updateItem.value);
      } else {
        return null;
      }
      dynamic body = {
        "fields": {content.key: content.value}
      };
      final response = await http.patch(
          _airtableSingleton._baseURL +
              "/$tableName/$id?api_key=${_airtableSingleton._apiKey}",
          body: json.encode(body),
          headers: {"Content-Type": "application/json"});
      return response;
    } else {
      //Please create the item first
      return null;
    }
  }

  ///Querying by an array of names which would match any of those.
  queryByNames(String tableName, String fieldName, List<String> values, {maxRecords: 100})async{
    String url = _airtableSingleton._baseURL+"/$tableName?api_key=${_airtableSingleton._apiKey}&filterByFormula=(OR(";
    for(int i = 0; i < values.length; i ++){
      url += "{$fieldName}=\"${values[i]}\"";
      if(!((i + 1)>=values.length))url+=",";
      }
    url+="))&maxRecords=$maxRecords";
    return await _query(url);
  }

  _query(url) async {
    final response = await http.get(url);
    final responseJson = json.decode(response.body);
    return responseJson;
  }
}
